/*
Carte du ciel

Copyright (C) 2013 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

/**
 * Carte du ciel
 */

function isCanvasSupported() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

function versionNavigateur() {
    document.body.appendChild(document.createTextNode("Cette simulation nécessite un navigateur récent."));
}

if (isCanvasSupported()) {
    new CarteDuCiel();
} else {
    if (window.addEventListener)
        window.addEventListener('load', function() { versionNavigateur(); }, false);
    else if (window.attachEvent)
        window.attachEvent('onload', function() { versionNavigateur(); });
    
}

function CarteDuCiel() {
    var date, params, p, i;
    
    this.applet = new SimuApplet(this);
    this.applet.appletDoc('carte_du_ciel_doc.xml');
    
    // pour changer la valeur par défaut de la date à partir de la date du jour
    this.applet.lireParametres();
    date = new Date();
    params = this.applet.getInParams();
    for (i=0; i<params.length; i++) {
        p = params[i];
        if (p.label == "jour")
            p.defaut = "" + date.getDate();
        else if (p.label == "mois")
            p.defaut = "" + (date.getMonth() + 1);
    }
    
    this.applet.setDelaiIterations(150);
    this.n = 192;
    
    this.phi = 48.8;
    this.jour = 1;
    this.mois = 1;
    
    this.canvas = null;
    this.ctx = null;
}

CarteDuCiel.prototype.initCalculLive = function(lin) {
    var affImage;
    
    this.phi = lin.lireDouble('phi') * Math.PI / 180;
    this.j = lin.lireDouble('jour');
    this.m = lin.lireDouble('mois');
    this.opt_horizon = ("on" == lin.lireString('opthorizon'));
    this.opt_equateur = ("on" == lin.lireString('optequateur'));
    this.opt_ecliptique = ("on" == lin.lireString('optecliptique'));
    this.opt_mois = ("on" == lin.lireString('optmois'));
    this.opt_etoiles = ("on" == lin.lireString('optetoiles'));
    this.opt_constellations = ("on" == lin.lireString('optconstellations'));
    
    this.etoiles = this.constructionEtoiles(this.lectureFichier("stars.txt"));
    this.constellations = this.constructionConstellations(this.lectureFichier("constellations.txt"));
    
    affImage = this.applet.getAffichageChoisi();
    this.imgdimx = affImage.imgdimx;
    this.imgdimy = affImage.imgdimy;
    this.margeAxeX = 30;
    this.margeAxeY = 30;
    this.largeurGraphique = this.imgdimx - this.margeAxeX;
    this.hauteurGraphique = this.imgdimy - this.margeAxeY;
    this.itps = 0;
    this.tps = 0;
    this.obli = 23.5 * Math.PI / 180.; // obliquité
    
    this.j += nbjours(this.m);
    this.lsol = (this.j - 81.) * 360. / 365.25 * Math.PI/180.; // long. ecli. du soleil
    
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', this.imgdimx);
    this.canvas.setAttribute('height', this.imgdimy);
    this.ctx = this.canvas.getContext('2d');
}

CarteDuCiel.prototype.calculLive = function() {
    var out, phi, cophi, lsol, obli, hsv, a, i, j, etoile, constellation, etoile2,
        x, y, sleve, d, metrics, h, vh, Az, dd;
    
    phi = this.phi;
    cophi = Math.PI * 0.5 - phi;
    lsol = this.lsol;
    obli = this.obli;
    
    this.ctx.fillStyle = 'white';
    this.ctx.fillRect(0, 0, this.imgdimx-1, this.imgdimy-1);
    
    hsv = this.tps * Math.PI / 12.;
    hsv = (hsv > 2 * Math.PI ? hsv - 2 * Math.PI : hsv);
    this.hsv = hsv;
    
    sleve = (leve(phi, lsol, hsv, this.obli) == 1);
    
    if (phi > 0 || !this.opt_horizon)
        this.ctx.fillStyle = '#106020';
    else
        this.ctx.fillStyle = sleve ? '#C0F0FF' : '#303070';
    this.ctx.fillRect(this.margeAxeX, 0, this.largeurGraphique, this.hauteurGraphique);
    
    this.ctx.font = '10px sans-serif';
    this.ctx.fillStyle = '#000000';
    this.ctx.fillText("h", this.margeAxeX + this.largeurGraphique / 2, this.hauteurGraphique + 24);
    for (a=0; a<=24; a+=1) {
        x = this.margeAxeX + a * this.largeurGraphique / 24.4;
        y = this.hauteurGraphique + 12;
        this.ctx.fillText(a, x - 4, y);
    }
    this.ctx.fillText("δ", this.margeAxeX - 26, this.hauteurGraphique / 2);
    for (d=-90; d<=90; d+=20) {
        metrics = this.ctx.measureText(d);
        x = this.margeAxeX - metrics.width - 2;
        y = (90 - d) * (this.hauteurGraphique - 6) / 180 + 3;
        this.ctx.fillText(d, x, y + 5);
    }
    
    // horizon
    if (this.opt_horizon) {
        // tracé de la zone du haut du graphique
        if (phi > 0)
            this.ctx.fillStyle = sleve ? '#C0F0FF' : '#303070';
        else
            this.ctx.fillStyle = '#106020';
        this.ctx.beginPath();
        for (a=-12; a<=12; a+=0.2) {
            x = this.xpix(a);
            y = delta_horizon(a * Math.PI / 12., cophi);
            y = this.ypix(y);
            if (a == -12)
                this.ctx.moveTo(x, y);
            else
                this.ctx.lineTo(x, y);
        }
        this.ctx.lineTo(this.margeAxeX + this.largeurGraphique, 0);
        this.ctx.lineTo(this.margeAxeX, 0);
        this.ctx.closePath();
        this.ctx.fill();
    }
    
    // équateur
    if (this.opt_equateur) {
        this.ctx.strokeStyle = '#FF0000';
        this.ctx.lineWidth = 2;
        this.ctx.beginPath();
        this.ctx.moveTo(this.margeAxeX, this.hauteurGraphique / 2);
        this.ctx.lineTo(this.margeAxeX + this.largeurGraphique, this.hauteurGraphique / 2);
        this.ctx.stroke();
    }
    
    // étoiles
    if (this.opt_etoiles) {
        for (i=0; i<this.etoiles.length; i++) {
            etoile = this.etoiles[i];
            alpha = etoile.alpha;
            delta = etoile.delta;
            h = ang_hor(hsv, lsol, alpha, obli) * 12/Math.PI;
            this.point(h, delta , '#FFA000', 2);
        }
    }
    // constellations
    if (this.opt_constellations) {
        this.ctx.strokeStyle = '#FFA000';
        this.ctx.lineWidth = 1;
        for (i=0; i<this.constellations.length; i++) {
            this.ctx.beginPath();
            constellation = this.constellations[i];
            etoile2 = null;
            for (j=0; j<constellation.length; j++) {
                etoile = constellation[j];
                alpha = etoile.alpha;
                delta = etoile.delta;
                //if (Math.abs(ang_hor(hsv, lsol, alpha, obli) * 12/Math.PI) >= 10)
                //    continue;
                h = ang_hor(hsv, lsol, alpha, obli) * 12/Math.PI;
                x = this.xpix(h);
                y = this.ypix(delta);
                if (j == 0)
                    this.ctx.moveTo(x, y);
                else {
                    if (j != 0 && Math.abs(h - vh) > 12)
                        this.ctx.moveTo(x, y);
                    this.ctx.lineTo(x, y);
                }
                vh = h;
            }
            this.ctx.stroke();
        }
    }
    
    // ecliptique
    if (this.opt_ecliptique) {
        this.ctx.beginPath();
        this.ctx.strokeStyle = '#FFFF00';
        this.ctx.lineWidth = 2;
        for (a=-12; a<=12; a++) {
            x = this.xpix(a);
            y = delta_ecl2(hsv, lsol, a * Math.PI / 12, this.obli);
            y = this.ypix(y);
            if (a == -12)
                this.ctx.moveTo(x, y);
            else
                this.ctx.lineTo(x, y);
        }
        this.ctx.stroke();
    }
    
    // le Soleil
    if (this.opt_ecliptique) {
        delta = delta_sol(lsol, obli);
        h = ang_hor(hsv, lsol, alpha_sol(lsol, obli), obli) * 12/Math.PI;
        this.point(h, delta, sleve ? '#FFFF00' : '#000000', 6);
    }
    
    // le premier de chaque mois
    if (this.opt_mois) {
        for (j=1; j<=12; j++)
            this.premier(j);
    }
    
    // points cardinaux
    if (this.opt_horizon) {
        Az = 0;
        dd = (phi > 0 ? -6 * Math.PI/180 : +6 * Math.PI/180);
        delta = delta_horizon(Az * Math.PI/12, cophi);
        this.point(Az, delta, '#000000', 3);
        delta += dd;
        this.texte("S", Az, delta, '#000000');
        
        Az = 6;
        delta = delta_horizon(Az * Math.PI/12, cophi);
        this.point(Az, delta, '#000000', 3);
        delta += dd;
        this.texte("O", Az, delta, '#000000');

        Az = 12;
        delta = delta_horizon(Az * Math.PI/12, cophi);
        this.point(Az, delta, '#000000', 3);
        delta += dd;
        this.texte("N", Az - 0.2, delta, '#000000');

        Az = -12;
        delta = delta_horizon(Az * Math.PI/12, cophi);
        this.point(Az, delta, '#000000', 3);
        delta += dd;
        this.texte("N", Az + 0.2, delta, '#000000');

        Az = -6;
        delta = delta_horizon(Az * Math.PI/12, cophi);
        this.point(Az, delta, '#000000', 3);
        delta += dd;
        this.texte("E", Az, delta, '#000000');
    }

    out = new ListeValeurs();
    out.ajouterString('infos', this.infos());
    out.ajouter('image', this.canvas);
    this.itps++;
    if (this.itps > this.n)
        this.itps -= this.n;
    this.tps = (this.itps * 24) / this.n;
    return out;
}

CarteDuCiel.prototype.premier = function(j) {
    var nj, l, alpha, delta;
    
    nj = 1 + nbjours(j);
    l = (nj - 81.) * 360./365.25 * Math.PI/180.; // long. ecli. du 1/1
    delta = delta_sol(l, this.obli);
    h = ang_hor(this.hsv, this.lsol, alpha_sol(l, this.obli), this.obli) * 12 / Math.PI;
    this.texte(j, h, delta + 0.1, '#0000FF');
    this.point(h, delta, '#0000FF', 3);
}

CarteDuCiel.prototype.infos = function() {
    return("heure solaire vraie = " + (this.hsv * 12 / Math.PI).toFixed(2)) + " h";
}

CarteDuCiel.prototype.xpix = function(h) {
    return(this.margeAxeX + (h + 12) * this.largeurGraphique / 24);
}

CarteDuCiel.prototype.ypix = function(delta) {
    return((90 - (delta * 180 / Math.PI)) * this.hauteurGraphique / 180);
}

CarteDuCiel.prototype.point = function(h, delta, couleur, rayon) {
    var x, y;
    this.ctx.fillStyle = couleur;
    this.ctx.beginPath();
    x = this.xpix(h);
    y = this.ypix(delta);
    this.ctx.arc(x, y, rayon, 0, 2*Math.PI, false);
    this.ctx.fill();
}

CarteDuCiel.prototype.texte = function(s, h, delta, couleur) {
    var x, y;
    this.ctx.font = '10px sans-serif';
    this.ctx.fillStyle = couleur;
    var metrics = this.ctx.measureText(s);
    x = this.xpix(h);
    y = this.ypix(delta);
    this.ctx.fillText(s, x - metrics.width / 2, y + 5);
}

CarteDuCiel.prototype.constructionEtoiles = function(lignes) {
    var etoiles, i, ligne, alpha, delta;
    
    etoiles = new Array();
    for (i=0; i<lignes.length; i++) {
        ligne = lignes[i];
        if (ligne.length >= 2) {
            alpha = parseFloat(ligne[0]);
            delta = parseFloat(ligne[1]);
            etoiles.push(new Etoile(alpha * Math.PI/12., delta * Math.PI/180.));
        }
    }
    return(etoiles);
}

CarteDuCiel.prototype.constructionConstellations = function(lignes) {
    var constellations, etoiles, heures, minutes, secondes, degres, alpha, delta;
    
    // en fait une "constellation" ici représente un sous-ensemble d'une constellation,
    // tracé par un ensemble d'étoiles
    constellations = new Array();
    etoiles = null;
    for (i=0; i<lignes.length; i++) {
        ligne = lignes[i];
        if (ligne.length == 2 && ligne[0].charAt(0) != '#') {
            if (etoiles == null)
                etoiles = new Array();
            alpha = parseFloat(ligne[0]);
            delta = parseFloat(ligne[1]);
            etoiles.push(new Etoile(alpha * Math.PI/12., delta * Math.PI/180.));
        } else if (etoiles != null) {
            constellations.push(etoiles);
            etoiles = null;
        }
    }
    return(constellations);
}

/**
 * Lecture d'un fichier ASCII utilisant des espaces et retours à la ligne comme séparateurs
 * Renvoie un tableau des lignes, chaque ligne étant un tableau de strings
 */
CarteDuCiel.prototype.lectureFichier = function(nomFichier) {
    var xmlhttp, slignes, tlignes, i, ligne, mots, ind1, j;
    
    xmlhttp = new XMLHttpRequest();
    if (xmlhttp.overrideMimeType)
        xmlhttp.overrideMimeType('text/plain');
    
    try {
        xmlhttp.open("GET", nomFichier, false);
        xmlhttp.send();
        slignes = xmlhttp.responseText.split("\n");
        tlignes = new Array(slignes.length);
        for (i=0; i<slignes.length; i++) {
            ligne = slignes[i];
            mots = new Array();
            
            ind1 = -1;
            for (j=0; j<ligne.length; j++) {
                if (ligne[j] != ' ' && ligne[j] != '\t') {
                    if (ind1 == -1)
                        ind1 = j;
                } else {
                    if (ind1 != -1) {
                        mots.push(ligne.substring(ind1, j));
                        ind1 = -1;
                    }
                }
            }
            if (ind1 != -1)
                mots.push(ligne.substring(ind1));
            
            tlignes[i] = mots;
        }
        return(tlignes);
    } catch (ex) {
        alert("Erreur à la lecture du fichier de données: " + ex);
        return(null);
    }
}

function Etoile(alpha, delta) {
    this.alpha = alpha;
    this.delta = delta;
}

function Constellation(tetoiles) {
    // tetoiles : tableau de tableaux d'étoiles
    this.tetoiles = tetoiles;
}

function nbjours(mois) {
    var nj;
    nj = 0;
    if (mois > 1)
        nj += 31;
    if (mois > 2)
        nj += 28;
    if (mois > 3)
        nj += 31;
    if (mois > 4)
        nj += 30;
    if (mois > 5)
        nj += 31;
    if (mois > 6)
        nj += 30;
    if (mois > 7)
        nj += 31;
    if (mois > 8)
        nj += 31;
    if (mois > 9)
        nj += 30;
    if (mois > 10)
        nj += 31;
    if (mois > 11)
        nj += 30;
    return(nj);
}

// modulo
function mod(x, y) {
    return(x - Math.floor(x / y) * y);
}

// place un angle entre -pi et pi
function pipi(x) {
    var modx2pi = mod(x, 2*Math.PI);
    return(modx2pi > Math.PI ?
        modx2pi - 2*Math.PI :
        (modx2pi < -Math.PI ?
            modx2pi + 2*Math.PI :
            modx2pi));
}

// coord equatorial d'un point de l'écliptique en fonction de la longitude écliptique:
function delta_ecl(l, obli) {
    return(Math.asin(Math.sin(l) * Math.sin(obli)));
}
function alpha_ecl(l, d) {
    return(Math.sin(l) > 0. ?
        Math.acos(Math.cos(l) / Math.cos(d)) :
        2. * Math.PI - Math.acos(Math.cos(l) / Math.cos(d)));
}

// coordonnées équatorial du Soleil

function delta_sol(lsol, obli) {
    return(delta_ecl(lsol, obli));
}
function alpha_sol(lsol, obli) {
    return(alpha_ecl(lsol, delta_sol(lsol, obli)));
}

// temps universel
function tu(hsv, lsol, obli) {
    return(hsv + alpha_sol(lsol, obli) - Math.PI);
}

// angle horaire en fonction de alpha et tu
function ang_hor(hsv, lsol, alpha, obli) {
    return(pipi(tu(hsv, lsol, obli) - alpha));
}

// asc. droite. en fonction de l'angle horaire et de tu
function asc_drt(hsv, lsol, h, obli) {
    return(tu(hsv, lsol, obli) - h);
}

// declinaison en fonction de alpha pour un point de l'écliptique
function delta_ecl2(hsv, lsol, h, obli) {
    return(Math.atan(Math.sin(asc_drt(hsv, lsol, h, obli))*Math.tan(obli)));
}

// déclinaison pour un point de l'horizon en fonction de l'angle horaire (le sud a un angle horaire égale à zéro et l'ouest à pi/2
function delta_horizon(x, cophi) {
    return(Math.atan(Math.sin(-Math.PI * 0.5 - x) * Math.tan(cophi)));
}

// détermine si le soleil est couché ou levé
function sinush(phi, lsol, hsv, obli) {
    return(Math.sin(phi) * Math.sin(delta_sol(lsol, obli)) + Math.cos(phi) *
        Math.cos(delta_sol(lsol, obli)) * Math.cos(hsv - Math.PI));
}
function leve(phi,lsol,hsv, obli) {
    return(sinush(phi, lsol, hsv, obli) > 0 ? 1 : 0); // 1 si levé, 0 si couché
}
